/* */

parse source . . exname .
parse arg args
if pos('/',args) > 0
   then do
      parse arg mask '/' args '(' opts
      args = '/'||strip(args)
   end
   else do
      parse arg mask '(' opts
      args = ''
   end
mask = strip(mask)

if mask = '?' | mask = '-?' then do
   say 'Enhanced directory command for a drive.  Displays entire drive contents.'
   say '  a file mask may be specified.  Defaults to *.*'
   say '  Use a mask of "." to start from the current relative subdirectory.'
   say 'Syntax:'
   say '  ' exname '<mask> </d:mm-dd-yy> </t:hr:min> <( DIR or FILES or BOTH>'
   say ' Where'
   say '   /d: signals a date comparison for greater than or equal to specified date.'
   say '        if /d: is given without a date, defaults to today.'
   say '        optionally, /de: for date equal to,'
   say '                 or /dl: for date less than or equal.'
   say '                 or /dg: for date greater than or equal.'
   say '   /t: signals a time comparison for greater than or equal to specified time.'
   say '        if /t: given without a date, defaults to today.'
   say '        optionally, /te: for time equal to,'
   say '                 or /tl: for time less than or equal.'
   say '                 or /tg: for time greater than or equal.'
   say '   /l /l+ display the .LONGNAME extended attribute as the file name'
   say '        instead of the actual short name on a VFAT file system'
   say '   /a: attributes must have.  For example /a:a-r for Archive not readonly'
   say '                                  or /a:hs for hidden and system files'

   say '  Default is to display files in ALL subdirectories'
   say '  Option DIR means only the directory tree in ALL subdirectories'
   say '  Option FILES means only Files in the one SPECIFIED subdirectory'
   exit 100
end

If mask = '' Then mask = '*'
If mask = '.' Then mask = mask || '\*'

what = 'F'      /* only files listed by default */
If left(translate(strip(opts)),3) = 'DIR'  Then what = 'D'
If left(translate(strip(opts)),4) = 'BOTH'  Then what = 'B'

nesting = 1   /* assume we will traverse all nested subdirectories */
If left(translate(strip(opts)),4) = 'FILE' Then nesting = 0

show_longname = 0
i = pos('/L',translate(args))
if i > 0 then do
   parse value substr(args,i) with _OptValue .
   show_longname = 1
   if length(_OptValue > 2) & right(_OptValue,1) = '+'
      then show_longname = 2
   args = delstr(args,i,length(_OptValue))
end

if pos('/d:',args) > 0 | pos('/de:',args) > 0 | pos('/dg:',args) > 0 | pos('/dl:',args) > 0 then do
   parse var args . '/d' comparator ':' begin_date . '/t' .
   comparator = translate(comparator)
   if begin_date = '' then do
      begin_date = date('U')
      say '   defaulting date to today =' begin_date
   end
   if pos('-',begin_date) > 0
      then parse var begin_date mm '-' dd '-' yy
      else  parse var begin_date mm '/' dd '/' yy
   if yy = '' then yy = left(date('S'),4)
   if yy <= 99 & yy >= 70
      then yy = yy + 1900
      else if yy < 100 then yy = yy + 2000
   begin_date = yy||'-'||right(100+mm,2)||'-'||right(100+dd,2)
   if (( mm < 1 | mm > 12) | (dd < 1 | dd > 31)) then do
      say '  Date entered was not in proper month-day-year form:'
      say '     a date of' begin_date 'is not appropriate.'
      exit 8
   end
end
else do
  begin_date = 0
  comparator = ''
end

parse var args . '/t' t_comparator ':' time_arg . '/' .
t_comparator = translate(t_comparator)
if time_arg = '' then time_arg = '00:00'
pm = 0
parse var time_arg hr ':' min ':' .
if min = '' then min = 0
if right(min,1) = 'p' then do
   pm = 12
   min = left(min,length(min)-1)
end
begin_time = ((hr + pm) * 60) + min

if begin_time > 0 & begin_date = 0
   then begin_date = substr(date('S'),1,4)||'-'||substr(date('S'),5,2)||'-'||substr(date('S'),7,2)

tAttrib = "*****"   /* ADHRS attribute mask for SysFileTree */
parse var args . '/a:' attributeFilter . 
do while attributeFilter \= '' 
   /* parse out the target attribute specification... i.e.  '/a:-r' for not read only */
   anAttr = left(attributeFilter,1)
   anAttrMask = '+'
   if anAttr = '-' then do
      anAttr = left(attributeFilter,2) 
      anAttrMask = '-'
   end
   attributeFilter = substr(attributeFilter,length(anAttr)+1)

   anAttr = translate(right(anAttr,1))
   maskPos = pos(anAttr,'ADHRS')

   tAttrib = overlay(anAttrMask,tAttrib,maskPos,1)
end

call RxFuncAdd "SysLoadFuncs", RexxUtil, "SysLoadFuncs"
call SysLoadFuncs

/* set up for recursion down the directory tree */
rootDir = filespec('drive',mask) || filespec('path',mask)
if rootDir = '' then rootDir = '.'
if filespec('path',mask) = '' & filespec('drive',mask) \= '' then do
   here = directory()
   rootDir = directory(rootDir||'.')
   here = directory(here)
end
mask = filespec('name',mask)

call listDirectory rootDir

exit

listDirectory: procedure expose mask begin_date begin_time comparator t_comparator show_longname nesting what tAttrib
parse arg rootDir
rootDir = strip(rootDir,'T','\')

/* list the contents this directory */
options = what||'L' /* 'L' returns date as YYYY-MM-DD HH:MM:SS */

rc = SysFileTree(rootDir || '\' || mask ,files,options,tAttrib)
Do i = 1 to files.0
  l = files.i
  parse var l dt tm size attr fname
  fname = strip(fname,'L')      /* remove any leading blanks */
  spot = pos(fname,l)-1

  shortName = filespec('name',fname)

  if show_longname > 0 then do
     rc = SysGetEA(fname,'.LONGNAME','longname')
     if (rc = 0 & length(longname) > 4) then do
        fname = filespec('drive',fname)||filespec('path',fname)||delstr(longname,1,4)
     end
     else shortName = ''
  end

  if begin_date > 0 then do
     if pos('/',dt) > 0 then do
        parse var dt mm '/' dd '/' yy
        if yy <= 99 & yy >= 70
           then yy = yy + 1900
           else yy = yy + 2000
        compare_date = yy||'-'||right(100+mm,2)||'-'||right(100+dd,2)
     end
     else do
        compare_date = dt
     end

     select
       when comparator = '' then do
            if compare_date < begin_date then iterate
         end
       when comparator = 'E' then do
            if compare_date \= begin_date then iterate
         end
       when comparator = 'G' then do
            if compare_date < begin_date then iterate
         end
       when comparator = 'L' then do
            if compare_date > begin_date then iterate
         end
       otherwise if compare_date < begin_date then iterate
     end

     pm = 0
     if right(tm,1) = 'p' then pm = 12
     parse value left(tm,length(tm)-1) with hr ':' min ':' sec
     minutes = ((hr + pm) * 60) + min

     select
       when t_comparator = '' then do
            /* interpret the time comparator using date rules */
            if compare_date = begin_date then do
               if comparator = 'L' then do
                  if minutes >= begin_time then iterate
               end
               else do
                  if minutes < begin_time then iterate
               end
            end
            /* else the time component is irrelevant as the date argument
               has already filtered the file's timestamp */
         end
       when t_comparator = 'E' then do
            if minutes \= begin_time then iterate
         end
       when t_comparator = 'G' then do
            if minutes < begin_time then iterate
         end
       when t_comparator = 'L' then do
            if minutes > begin_time then iterate
         end
       otherwise if minutes < begin_time then iterate
     end

  end

  /* reformat the output to enclose the file name in quotes in case of blanks */
  output = left(l,spot)||'"'||fname||'"'
  if show_longname > 1 then
     output = left(l,spot)|| left(shortName||"            ",12) '"'||fname||'"'
  say output

End

/* if to display the nested directory information */;
if nesting \= 0 then do
  rc = SysFileTree(rootDir || '\*',subDirs,'DO',tAttrib)

  Do d = 1 to subDirs.0
     call listDirectory subDirs.d
  End
End

return
